# ansisql.py
# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of SQLAlchemy and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

"""Defines ANSI SQL operations.

Contains default implementations for the abstract objects in the sql
module.
"""

from sqlalchemy import schema, sql, engine, util, sql_util, exceptions
from  sqlalchemy.engine import default
import string, re, sets, weakref, random

ANSI_FUNCS = sets.ImmutableSet(['CURRENT_DATE', 'CURRENT_TIME', 'CURRENT_TIMESTAMP',
                                'CURRENT_USER', 'LOCALTIME', 'LOCALTIMESTAMP',
                                'SESSION_USER', 'USER'])


RESERVED_WORDS = util.Set(['all', 'analyse', 'analyze', 'and', 'any', 'array',
                           'as', 'asc', 'asymmetric', 'authorization', 'between',
                           'binary', 'both', 'case', 'cast', 'check', 'collate',
                           'column', 'constraint', 'create', 'cross', 'current_date',
                           'current_role', 'current_time', 'current_timestamp',
                           'current_user', 'default', 'deferrable', 'desc',
                           'distinct', 'do', 'else', 'end', 'except', 'false',
                           'for', 'foreign', 'freeze', 'from', 'full', 'grant',
                           'group', 'having', 'ilike', 'in', 'initially', 'inner',
                           'intersect', 'into', 'is', 'isnull', 'join', 'leading',
                           'left', 'like', 'limit', 'localtime', 'localtimestamp',
                           'natural', 'new', 'not', 'notnull', 'null', 'off', 'offset',
                           'old', 'on', 'only', 'or', 'order', 'outer', 'overlaps',
                           'placing', 'primary', 'references', 'right', 'select',
                           'session_user', 'similar', 'some', 'symmetric', 'table',
                           'then', 'to', 'trailing', 'true', 'union', 'unique', 'user',
                           'using', 'verbose', 'when', 'where'])

LEGAL_CHARACTERS = util.Set(string.ascii_lowercase + string.ascii_uppercase + string.digits + '_$')
ILLEGAL_INITIAL_CHARACTERS = util.Set(string.digits + '$')

class ANSIDialect(default.DefaultDialect):
    def __init__(self, cache_identifiers=True, **kwargs):
        super(ANSIDialect,self).__init__(**kwargs)
        self.identifier_preparer = self.preparer()
        self.cache_identifiers = cache_identifiers

    def create_connect_args(self):
        return ([],{})

    def schemagenerator(self, *args, **kwargs):
        return ANSISchemaGenerator(self, *args, **kwargs)

    def schemadropper(self, *args, **kwargs):
        return ANSISchemaDropper(self, *args, **kwargs)

    def compiler(self, statement, parameters, **kwargs):
        return ANSICompiler(self, statement, parameters, **kwargs)

    def preparer(self):
        """Return an IdentifierPreparer.

        This object is used to format table and column names including
        proper quoting and case conventions.
        """
        return ANSIIdentifierPreparer(self)

class ANSICompiler(sql.Compiled):
    """Default implementation of Compiled.

    Compiles ClauseElements into ANSI-compliant SQL strings.
    """

    __traverse_options__ = {'column_collections':False}

    def __init__(self, dialect, statement, parameters=None, **kwargs):
        """Construct a new ``ANSICompiler`` object.

        dialect
          Dialect to be used

        statement
          ClauseElement to be compiled

        parameters
          optional dictionary indicating a set of bind parameters
          specified with this Compiled object.  These parameters are
          the *default* key/value pairs when the Compiled is executed,
          and also may affect the actual compilation, as in the case
          of an INSERT where the actual columns inserted will
          correspond to the keys present in the parameters.
        """
        
        sql.Compiled.__init__(self, dialect, statement, parameters, **kwargs)

        # if we are insert/update.  set to true when we visit an INSERT or UPDATE
        self.isinsert = self.isupdate = False
        
        # a dictionary of bind parameter keys to _BindParamClause instances.
        self.binds = {}
        
        # a dictionary of _BindParamClause instances to "compiled" names that are
        # actually present in the generated SQL
        self.bind_names = {}

        # a dictionary which stores the string representation for every ClauseElement
        # processed by this compiler.
        self.strings = {}

        # a dictionary which stores the string representation for ClauseElements
        # processed by this compiler, which are to be used in the FROM clause
        # of a select.  items are often placed in "froms" as well as "strings"
        # and sometimes with different representations.
        self.froms = {}

        # slightly hacky.  maps FROM clauses to WHERE clauses, and used in select
        # generation to modify the WHERE clause of the select.  currently a hack
        # used by the oracle module.
        self.wheres = {}

        # when the compiler visits a SELECT statement, the clause object is appended
        # to this stack.  various visit operations will check this stack to determine
        # additional choices (TODO: it seems to be all typemap stuff.  shouldnt this only
        # apply to the topmost-level SELECT statement ?)
        self.select_stack = []

        # a dictionary of result-set column names (strings) to TypeEngine instances,
        # which will be passed to a ResultProxy and used for resultset-level value conversion
        self.typemap = {}

        # a dictionary of select columns labels mapped to their "generated" label
        self.column_labels = {}

        # a dictionary of ClauseElement subclasses to counters, which are used to
        # generate truncated identifier names or "anonymous" identifiers such as
        # for aliases
        self.generated_ids = {}
        
        # True if this compiled represents an INSERT
        self.isinsert = False

        # True if this compiled represents an UPDATE
        self.isupdate = False

        # default formatting style for bind parameters
        self.bindtemplate = ":%s"

        # paramstyle from the dialect (comes from DBAPI)
        self.paramstyle = dialect.paramstyle

        # true if the paramstyle is positional
        self.positional = dialect.positional

        # a list of the compiled's bind parameter names, used to help
        # formulate a positional argument list
        self.positiontup = []

        # an ANSIIdentifierPreparer that formats the quoting of identifiers
        self.preparer = dialect.identifier_preparer

        # for UPDATE and INSERT statements, a set of columns whos values are being set
        # from a SQL expression (i.e., not one of the bind parameter values).  if present,
        # default-value logic in the Dialect knows not to fire off column defaults
        # and also knows postfetching will be needed to get the values represented by these
        # parameters.
        self.inline_params = None

    def after_compile(self):
        # this re will search for params like :param
        # it has a negative lookbehind for an extra ':' so that it doesnt match
        # postgres '::text' tokens
        match = re.compile(r'(?<!:):([\w_\$]+)', re.UNICODE)
        if self.paramstyle=='pyformat':
            self.strings[self.statement] = match.sub(lambda m:'%(' + m.group(1) +')s', self.strings[self.statement])
        elif self.positional:
            params = match.finditer(self.strings[self.statement])
            for p in params:
                self.positiontup.append(p.group(1))
            if self.paramstyle=='qmark':
                self.strings[self.statement] = match.sub('?', self.strings[self.statement])
            elif self.paramstyle=='format':
                self.strings[self.statement] = match.sub('%s', self.strings[self.statement])
            elif self.paramstyle=='numeric':
                i = [0]
                def getnum(x):
                    i[0] += 1
                    return str(i[0])
                self.strings[self.statement] = match.sub(getnum, self.strings[self.statement])

    def get_from_text(self, obj):
        return self.froms.get(obj, None)

    def get_str(self, obj):
        return self.strings[obj]

    def get_whereclause(self, obj):
        return self.wheres.get(obj, None)

    def construct_params(self, params):
        """Return a structure of bind parameters for this compiled object.

        This includes bind parameters that might be compiled in via
        the `values` argument of an ``Insert`` or ``Update`` statement
        object, and also the given `**params`.  The keys inside of
        `**params` can be any key that matches the
        ``BindParameterClause`` objects compiled within this object.

        The output is dependent on the paramstyle of the DBAPI being
        used; if a named style, the return result will be a dictionary
        with keynames matching the compiled statement.  If a
        positional style, the output will be a list, with an iterator
        that will return parameter values in an order corresponding to
        the bind positions in the compiled statement.

        For an executemany style of call, this method should be called
        for each element in the list of parameter groups that will
        ultimately be executed.
        """
        
        if self.parameters is not None:
            bindparams = self.parameters.copy()
        else:
            bindparams = {}
        bindparams.update(params)
        d = sql.ClauseParameters(self.dialect, self.positiontup)
        for b in self.binds.values():
            name = self.bind_names.get(b, b.key)
            d.set_parameter(b, b.value, name)

        for key, value in bindparams.iteritems():
            try:
                b = self.binds[key]
            except KeyError:
                continue
            name = self.bind_names.get(b, b.key)
            d.set_parameter(b, value, name)

        return d

    def default_from(self):
        """Called when a SELECT statement has no froms, and no FROM clause is to be appended.

        Gives Oracle a chance to tack on a ``FROM DUAL`` to the string output.
        """

        return ""
    
    def visit_grouping(self, grouping):
        self.strings[grouping] = "(" + self.strings[grouping.elem] + ")"
        
    def visit_label(self, label):
        labelname = self._truncated_identifier("colident", label.name)
        
        if len(self.select_stack):
            self.typemap.setdefault(labelname.lower(), label.obj.type)
            if isinstance(label.obj, sql._ColumnClause):
                self.column_labels[label.obj._label] = labelname
        self.strings[label] = self.strings[label.obj] + " AS "  + self.preparer.format_label(label, labelname)
        
    def visit_column(self, column):
        # there is actually somewhat of a ruleset when you would *not* necessarily
        # want to truncate a column identifier, if its mapped to the name of a 
        # physical column.  but thats very hard to identify at this point, and 
        # the identifier length should be greater than the id lengths of any physical
        # columns so should not matter.
        if not column.is_literal:
            name = self._truncated_identifier("colident", column.name)
        else:
            name = column.name

        if column.table is None or not column.table.named_with_column():
            self.strings[column] = self.preparer.format_column(column, name=name)
        else:
            if column.table.oid_column is column:
                n = self.dialect.oid_column_name(column)
                if n is not None:
                    self.strings[column] = "%s.%s" % (self.preparer.format_table(column.table, use_schema=False), n)
                elif len(column.table.primary_key) != 0:
                    pk = list(column.table.primary_key)[0]
                    pkname = (pk.is_literal and name or self._truncated_identifier("colident", pk.name))
                    self.strings[column] = self.preparer.format_column_with_table(list(column.table.primary_key)[0], column_name=pkname)
                else:
                    self.strings[column] = None
            else:
                self.strings[column] = self.preparer.format_column_with_table(column, column_name=name)

        if len(self.select_stack):
            # if we are within a visit to a Select, set up the "typemap"
            # for this column which is used to translate result set values
            self.typemap.setdefault(name.lower(), column.type)
            self.column_labels.setdefault(column._label, name.lower())

    def visit_fromclause(self, fromclause):
        self.froms[fromclause] = fromclause.name

    def visit_index(self, index):
        self.strings[index] = index.name

    def visit_typeclause(self, typeclause):
        self.strings[typeclause] = typeclause.type.dialect_impl(self.dialect).get_col_spec()

    def visit_textclause(self, textclause):
        self.strings[textclause] = textclause.text
        self.froms[textclause] = textclause.text
        if textclause.typemap is not None:
            self.typemap.update(textclause.typemap)

    def visit_null(self, null):
        self.strings[null] = 'NULL'

    def visit_clauselist(self, list):
        sep = list.operator
        if sep == ',':
            sep = ', '
        elif sep is None or sep == " ":
            sep = " "
        else:
            sep = " " + sep + " "
        self.strings[list] = string.join([s for s in [self.get_str(c) for c in list.clauses] if s is not None], sep)

    def apply_function_parens(self, func):
        return func.name.upper() not in ANSI_FUNCS or len(func.clauses) > 0

    def visit_calculatedclause(self, clause):
        self.strings[clause] = self.get_str(clause.clause_expr)

    def visit_cast(self, cast):
        if len(self.select_stack):
            # not sure if we want to set the typemap here...
            self.typemap.setdefault("CAST", cast.type)
        self.strings[cast] = "CAST(%s AS %s)" % (self.strings[cast.clause],self.strings[cast.typeclause])

    def visit_function(self, func):
        if len(self.select_stack):
            self.typemap.setdefault(func.name, func.type)
        if not self.apply_function_parens(func):
            self.strings[func] = ".".join(func.packagenames + [func.name])
            self.froms[func] = self.strings[func]
        else:
            self.strings[func] = ".".join(func.packagenames + [func.name]) + (not func.group and " " or "") + self.get_str(func.clause_expr)
            self.froms[func] = self.strings[func]

    def visit_compound_select(self, cs):
        text = string.join([self.get_str(c) for c in cs.selects], " " + cs.keyword + " ")
        group_by = self.get_str(cs.group_by_clause)
        if group_by:
            text += " GROUP BY " + group_by
        text += self.order_by_clause(cs)            
        text += self.visit_select_postclauses(cs)
        self.strings[cs] = text
        self.froms[cs] = "(" + text + ")"

    def visit_unary(self, unary):
        s = self.get_str(unary.element)
        if unary.operator:
            s = unary.operator + " " + s
        if unary.modifier:
            s = s + " " + unary.modifier
        self.strings[unary] = s
        
    def visit_binary(self, binary):
        result = self.get_str(binary.left)
        if binary.operator is not None:
            result += " " + self.binary_operator_string(binary)
        result += " " + self.get_str(binary.right)
        self.strings[binary] = result

    def binary_operator_string(self, binary):
        return binary.operator

    def visit_bindparam(self, bindparam):
        # apply truncation to the ultimate generated name

        if bindparam.shortname != bindparam.key:
            self.binds.setdefault(bindparam.shortname, bindparam)

        if bindparam.unique:
            count = 1
            key = bindparam.key

            # redefine the generated name of the bind param in the case
            # that we have multiple conflicting bind parameters.
            while self.binds.setdefault(key, bindparam) is not bindparam:
                tag = "_%d" % count
                key = bindparam.key + tag
                count += 1
            bindparam.key = key
            self.strings[bindparam] = self.bindparam_string(self._truncate_bindparam(bindparam))
        else:
            existing = self.binds.get(bindparam.key)
            if existing is not None and existing.unique:
                raise exceptions.CompileError("Bind parameter '%s' conflicts with unique bind parameter of the same name" % bindparam.key)
            self.strings[bindparam] = self.bindparam_string(self._truncate_bindparam(bindparam))
            self.binds[bindparam.key] = bindparam
    
    def _truncate_bindparam(self, bindparam):
        if bindparam in self.bind_names:
            return self.bind_names[bindparam]
            
        bind_name = bindparam.key
        if len(bind_name) > self.dialect.max_identifier_length():
            bind_name = self._truncated_identifier("bindparam", bind_name)
            # add to bind_names for translation
            self.bind_names[bindparam] = bind_name
        return bind_name
    
    def _truncated_identifier(self, ident_class, name):
        if (ident_class, name) in self.generated_ids:
            return self.generated_ids[(ident_class, name)]
        if len(name) > self.dialect.max_identifier_length():
            counter = self.generated_ids.get(ident_class, 1)
            truncname = name[0:self.dialect.max_identifier_length() - 6] + "_" + hex(counter)[2:]
            self.generated_ids[ident_class] = counter + 1
        else:
            truncname = name
        self.generated_ids[(ident_class, name)] = truncname
        return truncname
            
    def bindparam_string(self, name):
        return self.bindtemplate % name

    def visit_alias(self, alias):
        self.froms[alias] = self.get_from_text(alias.original) + " AS " + self.preparer.format_alias(alias)
        self.strings[alias] = self.get_str(alias.original)

    def visit_select(self, select):
        # the actual list of columns to print in the SELECT column list.
        inner_columns = util.OrderedDict()

        self.select_stack.append(select)
        for c in select._raw_columns:
            if hasattr(c, '_selectable'):
                s = c._selectable()
            else:
                self.traverse(c)
                inner_columns[self.get_str(c)] = c
                continue
            for co in s.columns:
                if select.use_labels:
                    labelname = co._label
                    if labelname is not None:
                        l = co.label(labelname)
                        self.traverse(l)
                        inner_columns[labelname] = l
                    else:
                        self.traverse(co)
                        inner_columns[self.get_str(co)] = co
                # TODO: figure this out, a ColumnClause with a select as a parent
                # is different from any other kind of parent
                elif select.is_selected_from and isinstance(co, sql._ColumnClause) and not co.is_literal and co.table is not None and not isinstance(co.table, sql.Select):
                    # SQLite doesnt like selecting from a subquery where the column
                    # names look like table.colname, so add a label synonomous with
                    # the column name
                    l = co.label(co.name)
                    self.traverse(l)
                    inner_columns[self.get_str(l.obj)] = l
                else:
                    self.traverse(co)
                    inner_columns[self.get_str(co)] = co
        self.select_stack.pop(-1)

        collist = string.join([self.get_str(v) for v in inner_columns.values()], ', ')

        text = "SELECT "
        text += self.visit_select_precolumns(select)
        text += collist

        whereclause = select.whereclause

        froms = []
        for f in select.froms:

            if self.parameters is not None:
                # TODO: whack this feature in 0.4
                # look at our own parameters, see if they
                # are all present in the form of BindParamClauses.  if
                # not, then append to the above whereclause column conditions
                # matching those keys
                for c in f.columns:
                    if sql.is_column(c) and self.parameters.has_key(c.key) and not self.binds.has_key(c.key):
                        value = self.parameters[c.key]
                    else:
                        continue
                    clause = c==value
                    if whereclause is not None:
                        whereclause = self.traverse(sql.and_(clause, whereclause), stop_on=util.Set([whereclause]))
                    else:
                        whereclause = clause
                        self.traverse(whereclause)

            # special thingy used by oracle to redefine a join
            w = self.get_whereclause(f)
            if w is not None:
                # TODO: move this more into the oracle module
                if whereclause is not None:
                    whereclause = self.traverse(sql.and_(w, whereclause), stop_on=util.Set([whereclause, w]))
                else:
                    whereclause = w

            t = self.get_from_text(f)
            if t is not None:
                froms.append(t)

        if len(froms):
            text += " \nFROM "
            text += string.join(froms, ', ')
        else:
            text += self.default_from()

        if whereclause is not None:
            t = self.get_str(whereclause)
            if t:
                text += " \nWHERE " + t

        group_by = self.get_str(select.group_by_clause)
        if group_by:
            text += " GROUP BY " + group_by

        if select.having is not None:
            t = self.get_str(select.having)
            if t:
                text += " \nHAVING " + t

        text += self.order_by_clause(select)
        text += self.visit_select_postclauses(select)
        text += self.for_update_clause(select)

        self.strings[select] = text
        self.froms[select] = "(" + text + ")"

    def visit_select_precolumns(self, select):
        """Called when building a ``SELECT`` statement, position is just before column list."""

        return select.distinct and "DISTINCT " or ""

    def visit_select_postclauses(self, select):
        """Called when building a ``SELECT`` statement, position is after all other ``SELECT`` clauses.

        Most DB syntaxes put ``LIMIT``/``OFFSET`` here.
        """

        return (select.limit or select.offset) and self.limit_clause(select) or ""

    def order_by_clause(self, select):
        order_by = self.get_str(select.order_by_clause)
        if order_by:
            return " ORDER BY " + order_by
        else:
            return ""

    def for_update_clause(self, select):
        if select.for_update:
            return " FOR UPDATE"
        else:
            return ""

    def limit_clause(self, select):
        text = ""
        if select.limit is not None:
            text +=  " \n LIMIT " + str(select.limit)
        if select.offset is not None:
            if select.limit is None:
                text += " \n LIMIT -1"
            text += " OFFSET " + str(select.offset)
        return text

    def visit_table(self, table):
        self.froms[table] = self.preparer.format_table(table)
        self.strings[table] = ""

    def visit_join(self, join):
        righttext = self.get_from_text(join.right)
        if join.right._group_parenthesized():
            righttext = "(" + righttext + ")"
        if join.isouter:
            self.froms[join] = (self.get_from_text(join.left) + " LEFT OUTER JOIN " + righttext +
            " ON " + self.get_str(join.onclause))
        else:
            self.froms[join] = (self.get_from_text(join.left) + " JOIN " + righttext +
            " ON " + self.get_str(join.onclause))
        self.strings[join] = self.froms[join]

    def visit_insert_column_default(self, column, default, parameters):
        """Called when visiting an ``Insert`` statement.

        For each column in the table that contains a ``ColumnDefault``
        object, add a blank *placeholder* parameter so the ``Insert``
        gets compiled with this column's name in its column and
        ``VALUES`` clauses.
        """

        parameters.setdefault(column.key, None)

    def visit_update_column_default(self, column, default, parameters):
        """Called when visiting an ``Update`` statement.

        For each column in the table that contains a ``ColumnDefault``
        object as an onupdate, add a blank *placeholder* parameter so
        the ``Update`` gets compiled with this column's name as one of
        its ``SET`` clauses.
        """

        parameters.setdefault(column.key, None)

    def visit_insert_sequence(self, column, sequence, parameters):
        """Called when visiting an ``Insert`` statement.

        This may be overridden compilers that support sequences to
        place a blank *placeholder* parameter for each column in the
        table that contains a Sequence object, so the Insert gets
        compiled with this column's name in its column and ``VALUES``
        clauses.
        """

        pass

    def visit_insert_column(self, column, parameters):
        """Called when visiting an ``Insert`` statement.

        This may be overridden by compilers who disallow NULL columns
        being set in an ``Insert`` where there is a default value on
        the column (i.e. postgres), to remove the column for which
        there is a NULL insert from the parameter list.
        """

        pass

    def visit_insert(self, insert_stmt):
        # scan the table's columns for defaults that have to be pre-set for an INSERT
        # add these columns to the parameter list via visit_insert_XXX methods
        default_params = {}
        class DefaultVisitor(schema.SchemaVisitor):
            def visit_column(s, c):
                self.visit_insert_column(c, default_params)
            def visit_column_default(s, cd):
                self.visit_insert_column_default(c, cd, default_params)
            def visit_sequence(s, seq):
                self.visit_insert_sequence(c, seq, default_params)
        vis = DefaultVisitor()
        for c in insert_stmt.table.c:
            if (isinstance(c, schema.SchemaItem) and (self.parameters is None or self.parameters.get(c.key, None) is None)):
                vis.traverse(c)

        self.isinsert = True
        colparams = self._get_colparams(insert_stmt, default_params)

        self.inline_params = util.Set()
        def create_param(col, p):
            if isinstance(p, sql._BindParamClause):
                self.binds[p.key] = p
                if p.shortname is not None:
                    self.binds[p.shortname] = p
                return self.bindparam_string(self._truncate_bindparam(p))
            else:
                self.inline_params.add(col)
                self.traverse(p)
                if isinstance(p, sql.ClauseElement) and not isinstance(p, sql.ColumnElement):
                    return "(" + self.get_str(p) + ")"
                else:
                    return self.get_str(p)

        text = ("INSERT INTO " + self.preparer.format_table(insert_stmt.table) + " (" + string.join([self.preparer.format_column(c[0]) for c in colparams], ', ') + ")" +
         " VALUES (" + string.join([create_param(*c) for c in colparams], ', ') + ")")

        self.strings[insert_stmt] = text

    def visit_update(self, update_stmt):
        # scan the table's columns for onupdates that have to be pre-set for an UPDATE
        # add these columns to the parameter list via visit_update_XXX methods
        default_params = {}
        class OnUpdateVisitor(schema.SchemaVisitor):
            def visit_column_onupdate(s, cd):
                self.visit_update_column_default(c, cd, default_params)
        vis = OnUpdateVisitor()
        for c in update_stmt.table.c:
            if (isinstance(c, schema.SchemaItem) and (self.parameters is None or self.parameters.get(c.key, None) is None)):
                vis.traverse(c)

        self.isupdate = True
        colparams = self._get_colparams(update_stmt, default_params)

        self.inline_params = util.Set()
        def create_param(col, p):
            if isinstance(p, sql._BindParamClause):
                self.binds[p.key] = p
                self.binds[p.shortname] = p
                return self.bindparam_string(self._truncate_bindparam(p))
            else:
                self.traverse(p)
                self.inline_params.add(col)
                if isinstance(p, sql.ClauseElement) and not isinstance(p, sql.ColumnElement):
                    return "(" + self.get_str(p) + ")"
                else:
                    return self.get_str(p)

        text = "UPDATE " + self.preparer.format_table(update_stmt.table) + " SET " + string.join(["%s=%s" % (self.preparer.format_column(c[0]), create_param(*c)) for c in colparams], ', ')

        if update_stmt.whereclause:
            text += " WHERE " + self.get_str(update_stmt.whereclause)

        self.strings[update_stmt] = text


    def _get_colparams(self, stmt, default_params):
        """Organize ``UPDATE``/``INSERT`` ``SET``/``VALUES`` parameters into a list of tuples.

        Each tuple will contain the ``Column`` and a ``ClauseElement``
        representing the value to be set (usually a ``_BindParamClause``,
        but could also be other SQL expressions.)

        The list of tuples will determine the columns that are
        actually rendered into the ``SET``/``VALUES`` clause of the
        rendered ``UPDATE``/``INSERT`` statement.  It will also
        determine how to generate the list/dictionary of bind
        parameters at execution time (i.e. ``get_params()``).

        This list takes into account the `values` keyword specified
        to the statement, the parameters sent to this Compiled
        instance, and the default bind parameter values corresponding
        to the dialect's behavior for otherwise unspecified primary
        key columns.
        """

        # no parameters in the statement, no parameters in the
        # compiled params - return binds for all columns
        if self.parameters is None and stmt.parameters is None:
            return [(c, sql.bindparam(c.key, type=c.type)) for c in stmt.table.columns]

        def to_col(key):
            if not isinstance(key, sql._ColumnClause):
                return stmt.table.columns.get(unicode(key), key)
            else:
                return key

        # if we have statement parameters - set defaults in the
        # compiled params
        if self.parameters is None:
            parameters = {}
        else:
            parameters = dict([(to_col(k), v) for k, v in self.parameters.iteritems()])

        if stmt.parameters is not None:
            for k, v in stmt.parameters.iteritems():
                parameters.setdefault(to_col(k), v)

        for k, v in default_params.iteritems():
            parameters.setdefault(to_col(k), v)

        # create a list of column assignment clauses as tuples
        values = []
        for c in stmt.table.columns:
            if parameters.has_key(c):
                value = parameters[c]
                if sql._is_literal(value):
                    value = sql.bindparam(c.key, value, type=c.type, unique=True)
                values.append((c, value))
        return values

    def visit_delete(self, delete_stmt):
        text = "DELETE FROM " + self.preparer.format_table(delete_stmt.table)

        if delete_stmt.whereclause:
            text += " WHERE " + self.get_str(delete_stmt.whereclause)

        self.strings[delete_stmt] = text

    def __str__(self):
        return self.get_str(self.statement)

class ANSISchemaBase(engine.SchemaIterator):
    def find_alterables(self, tables):
        alterables = []
        class FindAlterables(schema.SchemaVisitor):
            def visit_foreign_key_constraint(self, constraint):
                if constraint.use_alter and constraint.table in tables:
                    alterables.append(constraint)
        findalterables = FindAlterables()
        for table in tables:
            for c in table.constraints:
                findalterables.traverse(c)
        return alterables

class ANSISchemaGenerator(ANSISchemaBase):
    def __init__(self, dialect, connection, checkfirst=False, tables=None, **kwargs):
        super(ANSISchemaGenerator, self).__init__(connection, **kwargs)
        self.checkfirst = checkfirst
        self.tables = tables and util.Set(tables) or None
        self.preparer = dialect.preparer()
        self.dialect = dialect

    def get_column_specification(self, column, first_pk=False):
        raise NotImplementedError()

    def visit_metadata(self, metadata):
        collection = [t for t in metadata.table_iterator(reverse=False, tables=self.tables) if (not self.checkfirst or not self.dialect.has_table(self.connection, t.name, schema=t.schema))]
        for table in collection:
            table.accept_visitor(self)
        if self.dialect.supports_alter():
            for alterable in self.find_alterables(collection):
                self.add_foreignkey(alterable)

    def visit_table(self, table):
        for column in table.columns:
            if column.default is not None:
                column.default.accept_visitor(self)
            #if column.onupdate is not None:
            #    column.onupdate.accept_visitor(visitor)

        self.append("\nCREATE TABLE " + self.preparer.format_table(table) + " (")

        separator = "\n"

        # if only one primary key, specify it along with the column
        first_pk = False
        for column in table.columns:
            self.append(separator)
            separator = ", \n"
            self.append("\t" + self.get_column_specification(column, first_pk=column.primary_key and not first_pk))
            if column.primary_key:
                first_pk = True
            for constraint in column.constraints:
                constraint.accept_visitor(self)

        # On some DB order is significant: visit PK first, then the
        # other constraints (engine.ReflectionTest.testbasic failed on FB2)
        if len(table.primary_key):
            table.primary_key.accept_visitor(self)
        for constraint in [c for c in table.constraints if c is not table.primary_key]:
            constraint.accept_visitor(self)

        self.append("\n)%s\n\n" % self.post_create_table(table))
        self.execute()
        if hasattr(table, 'indexes'):
            for index in table.indexes:
                index.accept_visitor(self)

    def post_create_table(self, table):
        return ''

    def get_column_default_string(self, column):
        if isinstance(column.default, schema.PassiveDefault):
            if isinstance(column.default.arg, basestring):
                return "'%s'" % column.default.arg
            else:
                return str(self._compile(column.default.arg, None))
        else:
            return None

    def _compile(self, tocompile, parameters):
        """compile the given string/parameters using this SchemaGenerator's dialect."""
        compiler = self.dialect.compiler(tocompile, parameters)
        compiler.compile()
        return compiler

    def visit_check_constraint(self, constraint):
        self.append(", \n\t")
        if constraint.name is not None:
            self.append("CONSTRAINT %s " %
                        self.preparer.format_constraint(constraint))
        self.append(" CHECK (%s)" % constraint.sqltext)

    def visit_column_check_constraint(self, constraint):
        self.append(" CHECK (%s)" % constraint.sqltext)

    def visit_primary_key_constraint(self, constraint):
        if len(constraint) == 0:
            return
        self.append(", \n\t")
        if constraint.name is not None:
            self.append("CONSTRAINT %s " % self.preparer.format_constraint(constraint))
        self.append("PRIMARY KEY ")
        self.append("(%s)" % (string.join([self.preparer.format_column(c) for c in constraint],', ')))

    def visit_foreign_key_constraint(self, constraint):
        if constraint.use_alter and self.dialect.supports_alter():
            return
        self.append(", \n\t ")
        self.define_foreign_key(constraint)

    def add_foreignkey(self, constraint):
        self.append("ALTER TABLE %s ADD " % self.preparer.format_table(constraint.table))
        self.define_foreign_key(constraint)
        self.execute()

    def define_foreign_key(self, constraint):
        preparer = self.preparer
        if constraint.name is not None:
            self.append("CONSTRAINT %s " %
                        preparer.format_constraint(constraint))
        self.append("FOREIGN KEY(%s) REFERENCES %s (%s)" % (
            string.join([preparer.format_column(f.parent) for f in constraint.elements], ', '),
            preparer.format_table(list(constraint.elements)[0].column.table),
            string.join([preparer.format_column(f.column) for f in constraint.elements], ', ')
        ))
        if constraint.ondelete is not None:
            self.append(" ON DELETE %s" % constraint.ondelete)
        if constraint.onupdate is not None:
            self.append(" ON UPDATE %s" % constraint.onupdate)

    def visit_unique_constraint(self, constraint):
        self.append(", \n\t")
        if constraint.name is not None:
            self.append("CONSTRAINT %s " %
                        self.preparer.format_constraint(constraint))
        self.append(" UNIQUE (%s)" % (string.join([self.preparer.format_column(c) for c in constraint],', ')))

    def visit_column(self, column):
        pass

    def visit_index(self, index):
        preparer = self.preparer        
        self.append('CREATE ')
        if index.unique:
            self.append('UNIQUE ')
        self.append('INDEX %s ON %s (%s)' \
                    % (preparer.format_index(index),
                       preparer.format_table(index.table),
                       string.join([preparer.format_column(c) for c in index.columns], ', ')))
        self.execute()

class ANSISchemaDropper(ANSISchemaBase):
    def __init__(self, dialect, connection, checkfirst=False, tables=None, **kwargs):
        super(ANSISchemaDropper, self).__init__(connection, **kwargs)
        self.checkfirst = checkfirst
        self.tables = tables
        self.preparer = dialect.preparer()
        self.dialect = dialect

    def visit_metadata(self, metadata):
        collection = [t for t in metadata.table_iterator(reverse=True, tables=self.tables) if (not self.checkfirst or  self.dialect.has_table(self.connection, t.name, schema=t.schema))]
        if self.dialect.supports_alter():
            for alterable in self.find_alterables(collection):
                self.drop_foreignkey(alterable)
        for table in collection:
            table.accept_visitor(self)

    def visit_index(self, index):
        self.append("\nDROP INDEX " + self.preparer.format_index(index))
        self.execute()

    def drop_foreignkey(self, constraint):
        self.append("ALTER TABLE %s DROP CONSTRAINT %s" % (
            self.preparer.format_table(constraint.table),
            self.preparer.format_constraint(constraint)))
        self.execute()

    def visit_table(self, table):
        for column in table.columns:
            if column.default is not None:
                column.default.accept_visitor(self)

        self.append("\nDROP TABLE " + self.preparer.format_table(table))
        self.execute()

class ANSIDefaultRunner(engine.DefaultRunner):
    pass

class ANSIIdentifierPreparer(object):
    """Handle quoting and case-folding of identifiers based on options."""

    def __init__(self, dialect, initial_quote='"', final_quote=None, omit_schema=False):
        """Construct a new ``ANSIIdentifierPreparer`` object.

        initial_quote
          Character that begins a delimited identifier.

        final_quote
          Character that ends a delimited identifier. Defaults to `initial_quote`.

        omit_schema
          Prevent prepending schema name. Useful for databases that do
          not support schemae.
        """

        self.dialect = dialect
        self.initial_quote = initial_quote
        self.final_quote = final_quote or self.initial_quote
        self.omit_schema = omit_schema
        self.__strings = {}

    def _escape_identifier(self, value):
        """Escape an identifier.

        Subclasses should override this to provide database-dependent
        escaping behavior.
        """

        return value.replace('"', '""')

    def quote_identifier(self, value):
        """Quote an identifier.

        Subclasses should override this to provide database-dependent
        quoting behavior.
        """

        return self.initial_quote + self._escape_identifier(value) + self.final_quote

    def _fold_identifier_case(self, value):
        """Fold the case of an identifier.

        Subclasses should override this to provide database-dependent
        case folding behavior.
        """

        return value
        # ANSI SQL calls for the case of all unquoted identifiers to be folded to UPPER.
        # some tests would need to be rewritten if this is done.
        #return value.upper()

    def _reserved_words(self):
        return RESERVED_WORDS

    def _legal_characters(self):
        return LEGAL_CHARACTERS

    def _illegal_initial_characters(self):
        return ILLEGAL_INITIAL_CHARACTERS

    def _requires_quotes(self, value, case_sensitive):
        """Return True if the given identifier requires quoting."""
        return \
            value in self._reserved_words() \
            or (value[0] in self._illegal_initial_characters()) \
            or bool(len([x for x in unicode(value) if x not in self._legal_characters()])) \
            or (case_sensitive and value.lower() != value)

    def __generic_obj_format(self, obj, ident):
        if getattr(obj, 'quote', False):
            return self.quote_identifier(ident)
        if self.dialect.cache_identifiers:
            case_sens = getattr(obj, 'case_sensitive', None)
            try:
                return self.__strings[(ident, case_sens)]
            except KeyError:
                if self._requires_quotes(ident, getattr(obj, 'case_sensitive', ident == ident.lower())):
                    self.__strings[(ident, case_sens)] = self.quote_identifier(ident)
                else:
                    self.__strings[(ident, case_sens)] = ident
                return self.__strings[(ident, case_sens)]
        else:
            if self._requires_quotes(ident, getattr(obj, 'case_sensitive', ident == ident.lower())):
                return self.quote_identifier(ident)
            else:
                return ident

    def should_quote(self, object):
        return object.quote or self._requires_quotes(object.name, object.case_sensitive)

    def is_natural_case(self, object):
        return object.quote or self._requires_quotes(object.name, object.case_sensitive)

    def format_sequence(self, sequence):
        return self.__generic_obj_format(sequence, sequence.name)

    def format_label(self, label, name=None):
        return self.__generic_obj_format(label, name or label.name)

    def format_alias(self, alias):
        return self.__generic_obj_format(alias, alias.name)

    def format_constraint(self, constraint):
        return self.__generic_obj_format(constraint, constraint.name)

    def format_index(self, index):
        return self.__generic_obj_format(index, index.name)

    def format_table(self, table, use_schema=True, name=None):
        """Prepare a quoted table and schema name."""

        if name is None:
            name = table.name
        result = self.__generic_obj_format(table, name)
        if use_schema and getattr(table, "schema", None):
            result = self.__generic_obj_format(table, table.schema) + "." + result
        return result

    def format_column(self, column, use_table=False, name=None):
        """Prepare a quoted column name."""
        if name is None:
            name = column.name
        if not getattr(column, 'is_literal', False):
            if use_table:
                return self.format_table(column.table, use_schema=False) + "." + self.__generic_obj_format(column, name)
            else:
                return self.__generic_obj_format(column, name)
        else:
            # literal textual elements get stuck into ColumnClause alot, which shouldnt get quoted
            if use_table:
                return self.format_table(column.table, use_schema=False) + "." + name
            else:
                return name

    def format_column_with_table(self, column, column_name=None):
        """Prepare a quoted column name with table name."""
        
        return self.format_column(column, use_table=True, name=column_name)

dialect = ANSIDialect



# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovie7arte.py 478 2006-12-05 21:14:51Z piotrek $'

# Copyright (c) 2005-2007 Vasco Nunes, Piotr Ozarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string

plugin_name = "Cineteka"
plugin_description = "O seu Clube de Video Online"
plugin_url = "cineteka.com"
plugin_language = _("Portuguese")
plugin_author = "Vasco Nunes"
plugin_author_email = "<vasco.m.nunes@gmail.com>"
plugin_version = "0.3"

class Plugin(movie.Movie):
	"""A movie plugin object"""
	def __init__(self, id):
		self.encode='iso-8859-1'
		self.movie_id = id
		self.url = "http://www.cineteka.com/index.php?op=Movie&id=" + str(self.movie_id)

	def get_image(self):
		"""Finds the film's poster image"""
		self.image_url = "http://www.cineteka.com/img/filmes/" + str(self.movie_id) + "_big.jpg"

	def get_o_title(self):
		"""Finds the film's original title"""
		self.o_title = string.capwords(gutils.trim(self.page, """<nobr><span class="txt11">(""", ")"))

	def get_title(self):
		"""Finds the film's local title.
		Probably the original title translation"""
		self.title = string.capwords(gutils.trim(self.page, """<td class="txt12"><b>""", "</b>"))

	def get_director(self):
		"""Finds the film's director"""
		self.director = gutils.strip_tags(gutils.trim(self.page, "<div><b>Realização:</b><br>", "</div>"))
		self.director = string.replace(self.director,"<br>", ", ")
		self.director = gutils.strip_tags(self.director)

	def get_plot(self):
		"""Finds the film's plot"""
		self.plot = gutils.trim(self.page, """<div class="txt-normal" style="margin-top: 10px;">""", "</td></tr></table>")

	def get_year(self):
		"""Finds the film's year"""
		self.year = gutils.trim(self.page, "<b>Ano:</b> ", "</div>")

	def get_runtime(self):
		"""Finds the film's running time"""
		self.runtime = gutils.trim(self.page, "<td><b>Duração:</b> ", " min.</td>")

	def get_genre(self):
		"""Finds the film's genre"""
		self.genre = gutils.strip_tags(gutils.trim(self.page, "<b>Género:</b><br>", "</div>"))
		self.genre = string.replace(self.genre,"<br>", ", ")
		self.genre = gutils.strip_tags(self.genre)

	def get_cast(self):
		self.cast = gutils.strip_tags(gutils.trim(self.page, "<b>Elenco:</b> ", "</td>"))
		self.cast = string.replace(self.cast, ", ", "")
		self.cast = string.replace(self.cast, "\t", "")
		self.cast = string.replace(self.cast, "\n ", "\n")

	def get_classification(self):
		"""Find the film's classification"""
		self.classification = gutils.trim(self.page, "<b>Idade:</b> ", "</div>")

	def get_studio(self):
		"""Find the studio"""
		self.studio = ""

	def get_o_site(self):
		"""Find the film's oficial site"""
		self.o_site = gutils.trim(self.page, \
			"<a class=\"button\" href=\"", \
			" title=\"Consultar título no Internet Movie Data Base\"")

	def get_site(self):
		"""Find the film's imdb details page"""
		self.site = gutils.trim(self.page, \
			"</td><td><a class=\"button\" href=\"", \
			"""" title="Consultar tÃ­tulo""")

	def get_trailer(self):
		"""Find the film's trailer page or location"""
		self.trailer = ""

	def get_country(self):
		"""Find the film's country"""
		self.country = gutils.trim(self.page, "<b>País:</b><br>", "</div>")
		self.country = string.replace(self.country,"<br>", ", ")
		self.country = gutils.strip_tags(self.country)

	def get_rating(self):
		"""Find the film's rating. From 0 to 10.
		Convert if needed when assigning."""
		self.rating = 0

class SearchPlugin(movie.SearchMovie):
	"""A movie search object"""
	def __init__(self):
		self.original_url_search = "http://www.cineteka.com/index.php?op=MovieSearch&s="
		self.translated_url_search = self.original_url_search
		self.encode='iso-8859-1'

	def search(self, parent_window):
		"""Perform the web search"""
		self.open_search(parent_window)
		self.sub_search()
		return self.page

	def sub_search(self):
		"""Isolating just a portion (with the data we want) of the results"""
		self.page = gutils.trim(self.page, \
			"""o:</div>""", """<div style="margin-top: 10px; text-align: center;"></div>""")

	def get_searches(self):
		"""Try to find both id and film title for each search result"""
		elements = string.split(self.page, "</a>]</td>")
		self.number_results = elements[-1]

		if (elements[0]<>''):
			for element in elements:
				self.ids.append(gutils.trim(element, "?op=Movie&id=", "\""))
				self.titles.append(gutils.strip_tags(gutils.convert_entities \
					(gutils.trim(element, """<td class="txt12"><b>""", "</span></nobr></td>"))))
		else:
			self.number_results = 0




# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovieFilmeVonAZ.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2006-2007 Michael Jahn
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string
import re

plugin_name = "FilmeVonA-Z.de"
plugin_description = "FILMEvonA-Z.de"
plugin_url = "www.filmevona-z.de"
plugin_language = _("German")
plugin_author = "Michael Jahn"
plugin_author_email = "<mikej06@hotmail.com>"
plugin_version = "1.3"

class Plugin(movie.Movie):
	def __init__(self, id):
		self.encode='utf-8'
		self.movie_id = id
		self.url = 'http://www.filmevona-z.de/filmsuche.cfm?sucheNach=Titel&wert=' + str(self.movie_id)

	def get_image(self):
		self.image_url = gutils.trim(self.page, 'ProductCover=', '"')
		if not self.image_url == '':
			self.image_url = "http://www.filmevona-z.de/" + self.image_url

	def get_o_title(self):
		self.o_title = string.capwords(
			gutils.clean(gutils.after(
			self.regextrim(self.page, '"[ \t]+class="text_ergebniss_titel"', '[ \t]+[(]Originaltitel[)]'), '</a>')))
		if self.o_title == '':
			self.o_title = gutils.after(gutils.trim(self.page, 'sucheNach=titel', '</a>'), '>')

	def get_title(self):
		self.title = gutils.after(gutils.trim(self.page, 'sucheNach=titel', '</a>'), '>')

	def get_director(self):
		self.director = gutils.after(gutils.trim(self.page, '(Regie)', '</a>'), '>')

	def get_plot(self):
		self.plot = gutils.after(self.regextrim(self.page, '[(]Darsteller[)]', '</[pP]>'), '<p>')

	def get_year(self):
		self.year = gutils.after(gutils.trim(self.page, 'sucheNach=produktionsjahr', '</a>'), '>')

	def get_runtime(self):
		self.runtime = gutils.trim(self.page, 'L&auml;nge: ', ' Minuten')

	def get_genre(self):
		elements = string.split(self.page, 'sucheNach=genre')
		if (elements[0]<>''):
			elements[0] = ''
			self.delimiter = ''
			self.genre = ''
			for element in elements:
				if (element <> ''):
					self.genre += self.delimiter + gutils.trim(element, '>', '</a>')
					self.delimiter = ", "

	def get_cast(self):
		self.cast = self.regextrim(self.page, '[(]Darsteller[)]', '(<[pP]>|<br><span[^>]+>)')
		self.cast = gutils.clean(self.cast)
		self.cast = self.cast.replace(' als ', _(' as '))
		self.cast = re.sub('( \t|\t|\r|\n)', '', self.cast)
		self.cast = self.cast.replace(', ', '\n')
		self.cast = self.cast.replace(',', '')

	def get_classification(self):
		self.classification = self.regextrim(self.page, 'FSK:[ ]+', '[,;]')

	def get_studio(self):
		self.studio = gutils.after(gutils.trim(self.page, 'sucheNach=produktionsfirma', '</a>'), '>')

	def get_o_site(self):
		self.o_site = ''

	def get_site(self):
		self.site = 'http://www.filmevona-z.de/filmsuche.cfm?sucheNach=Titel&wert=' + self.movie_id;

	def get_trailer(self):
		self.trailer = ''

	def get_country(self):
		self.country = gutils.after(gutils.trim(self.page, 'sucheNach=produktionsland', '</a>'), '>')

	def get_rating(self):
		self.rating = 0
		
	def regextrim(self,text,key1,key2):
		obj = re.search(key1, text)
		if obj is None:
			return ''
		else:
			p1 = obj.end()
		obj = re.search(key2, text[p1:])
		if obj is None:
			return ''
		else:
			p2 = p1 + obj.start()
		return text[p1:p2]

class SearchPlugin(movie.SearchMovie):
	def __init__(self):
		self.original_url_search	= "http://www.filmevona-z.de/filmsuche.cfm?sucheNach=Titel&wert="
		self.translated_url_search	= "http://www.filmevona-z.de/filmsuche.cfm?sucheNach=Titel&wert="
		self.encode='utf-8'

	def search(self,parent_window):
		self.open_search(parent_window)
		# used for looking for subpages
		tmp_page = gutils.trim(self.page, "Treffer-Seite", "chste Seite")
		elements = string.split(tmp_page, '" class="text_navi">')
		# first results
		tmp_page = gutils.after(gutils.trim(self.page,"Alle Treffer aus der Kategorie", "Treffer-Seite"), "Titel:")
		# look for subpages
		for element in elements:
			element = gutils.before(element, "</a>")
			try:
				tmp_element = int(element)
			except:
				tmp_element = 1
			if (tmp_element <> 1):
				self.url = "http://www.filmevona-z.de/filmsuche.cfm?sucheNach=Titel&currentPage=" + str(tmp_element) + "&wert="
				self.open_search(parent_window)
				tmp_page2 = gutils.trim(self.page,"Alle Treffer aus der Kategorie", "Treffer-Seite")
				tmp_page = tmp_page + tmp_page2
		self.page = tmp_page

		return self.page

	def get_searches(self):
		elements = string.split(self.page,'class="text_ergebniss_titel"')
		i = 0
		while i < len(elements) - 1:
			id_part = elements[i]
			i = i + 1
			text_part = elements[i]
			i = i + 1
			self.ids.append(gutils.trim(id_part, 'filmsuche.cfm?wert=', '&'))
			self.titles.append(gutils.strip_tags(
						gutils.trim(text_part, '>', '</a>') + ' (' +
						string.capwords(gutils.trim(text_part, '</a>', '(Originaltitel)')) + ', ' +
						gutils.after(gutils.trim(text_part, 'sucheNach=produktionsland', '</a>'), '>') + ', ' +
						gutils.after(gutils.trim(text_part, 'sucheNach=produktionsjahr', '</a>'), '>') +
						')'))

#
# Plugin Test
#
class SearchPluginTest(SearchPlugin):
	#
	# Configuration for automated tests:
	# dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
	#
	test_configuration = {
		'Rocky Balboa'			: [ 1, 1 ],
		'Arahan'				: [ 1, 1 ],
		'Ein glückliches Jahr'	: [ 0, 0 ]
	}

class PluginTest:
	#
	# Configuration for automated tests:
	# dict { movie_id -> dict { arribute -> value } }
	#
	# value: * True/False if attribute should only be tested for any value
	#        * or the expected value
	#
	test_configuration = {
		'528267' : { 
			'title' 			: 'Rocky Balboa',
			'o_title' 			: 'Rocky Balboa',
			'director'			: 'Sylvester Stallone',
			'plot' 				: True,
			'cast'				: 'A.J. Benza' + _(' as ') + 'L.C.\n\
Milo Ventimiglia' + _(' as ') + 'Rocky jr.\n\
Antonio Tarver' + _(' as ') + 'Mason \'The Line\' Dixon\n\
Geraldine Hughes' + _(' as ') + 'Marie\n\
Sylvester Stallone' + _(' as ') + 'Rocky Balboa\n\
Burt Young' + _(' as ') + 'Paulie\n\
Tony Burton' + _(' as ') + 'Duke',
			'country'			: 'USA',
			'genre'				: 'Boxerfilm',
			'classification'	: 'ab 12',
			'studio'			: 'Columbia Pic./ MGM/ Rogue Marble/ Revolution Studios/ Chartoff-Winkler Prod.',
			'o_site'			: False,
			'site'				: 'http://www.filmevona-z.de/filmsuche.cfm?sucheNach=Titel&wert=528267',
			'trailer'			: False,
			'year'				: 2006,
			'notes'				: False,
			'runtime'			: 102,
			'image'				: False,
			'rating'			: False
		},
		'26956' : { 
			'title' 			: 'Bürgschaft für ein Jahr',
			'o_title' 			: 'Bürgschaft für ein Jahr',
			'director'			: 'Herrmann Zschoche',
			'plot' 				: True,
			'cast'				: 'Heide Kipp' + _(' as ') + 'Frau Braun\n\
Jan Spitzer' + _(' as ') + 'Werner Horn\n\
Monika Lennartz' + _(' as ') + 'Irmgard Behrend\n\
Katrin Saß' + _(' as ') + 'Nina\n\
Ursula Werner' + _(' as ') + 'Frau Müller\n\
Christian Steyer' + _(' as ') + 'Heiner Menk\n\
Jaecki Schwarz' + _(' as ') + 'Peter Müller\n\
Barbara Dittus' + _(' as ') + 'Heimleiterin',
			'country'			: 'DDR',
			'genre'				: 'Arbeiterfilm, Frauenfilm, Literaturverfilmung',
			'classification'	: 'ab 6',
			'studio'			: 'DEFA, Gruppe "Berlin"',
			'o_site'			: False,
			'site'				: 'http://www.filmevona-z.de/filmsuche.cfm?sucheNach=Titel&wert=26956',
			'trailer'			: False,
			'year'				: 1981,
			'notes'				: False,
			'runtime'			: 93,
			'image'				: False,
			'rating'			: False
		},
		'524017' : { 
			'title' 			: 'Arahan',
			'o_title' 			: 'Arahan Jangpung Daejakjeon',
			'director'			: 'Ryoo Seung-wan',
			'plot' 				: True,
			'cast'				: 'Yoon So-yi' + _(' as ') + 'Wi-jin\n\
Yun Ju-sang' + _(' as ') + 'Mu-woon\n\
Ahn Sung-kee' + _(' as ') + 'Ja-woon\n\
Jung Doo-hong' + _(' as ') + 'Heukwoon\n\
Ryu Seung-beom' + _(' as ') + 'Sang-hwan',
			'country'			: 'Südkorea',
			'genre'				: False,
			'classification'	: 'ab 16',
			'studio'			: 'Fun and Happiness/ Good Movie',
			'o_site'			: False,
			'site'				: 'http://www.filmevona-z.de/filmsuche.cfm?sucheNach=Titel&wert=524017',
			'trailer'			: False,
			'year'				: 2004,
			'notes'				: False,
			'runtime'			: 108,
			'image'				: False,
			'rating'			: False
		}
	}



# -*- coding: ISO-8859-1 -*-

__revision__ = '$Id: gconsole.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2005-2008 Vasco Nunes, Piotr Ożarowski

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import getopt
import sys
import gutils

options = ('hDCo:t:d:c:y:s:', ('help', 'debug', 'sqlecho', 'clean', 'check-dep',
	'show-dep', 'original_title=', 'title=', 'director=', 'cast=', 'year=',
	'sort=', 'seen=', 'loaned=', 'number=', 'runtime=', 'rating=', 'home=',
	'config='))

def check_args(self):
	self._tmp_home = None
	self._tmp_config = 'griffith.cfg'
	if len(sys.argv)>1:
		try:
			opts, args = getopt.getopt(sys.argv[1:], options[0], options[1])
		except getopt.GetoptError:
			# print help information and exit:
			con_usage()
			sys.exit(2)
		
		for o, a in opts:
			if o in ('-h', '--help'):
				con_usage()
				sys.exit()
			elif o in ('-D', '--debug'):
				self.debug.set_debug()
			elif o == '--home':
				self._tmp_home = a # see initialize.locations()
			elif o == '--config':
				self._tmp_config = a # griffith.__init__
			elif o == '--check-dep':
				check_dependencies()
				sys.exit()
			elif o == '--show-dep':
				show_dependencies()
				sys.exit()

def check_args_with_db(self):
	if len(sys.argv)>1:
		try:
			opts, args = getopt.getopt(sys.argv[1:], options[0], options[1])
		except getopt.GetoptError:
			# print help information and exit:
			con_usage()
			sys.exit(3)

		sort = None
		where = {}
		for o, a in opts:
			if o in ('-C', '--clean'):
				gutils.clean_posters_dir(self)
				sys.exit()
			elif o == '--sqlecho':
				self.db.metadata.engine.echo = True
			elif o in ('-s', '--sort'):
				sort = a
			elif o in ('-o', '--original_title'):
				where['o_title'] = a
			elif o in ('-t', '--title'):
				where['title'] = a
			elif o in ('-d', '--director'):
				where['director'] = a
			elif o in ('-c', '--cast'):
				where['cast'] = a
			elif o in ('-y', '--year'):
				where['year'] = str(int(a))
			elif o == '--seen':
				where['seen'] = a
			elif o == '--loaned':
				where['loaned'] = a
			elif o == '--number':
				where['number'] = a
			elif o == '--runtime':
				where['runtime'] = a
			elif o == '--rating':
				where['rating'] = a
		if where:
			con_search_movie(self, where, sort)

def con_search_movie(self, where, sort=None):
	# for search function
	from sqlalchemy import select
	col = lambda x: self.db.Movie.c[x]
	columns = (col('number'), col('title'), col('o_title'), col('director'), col('year'))
	
	sort_columns = []
	if sort:
		for i in sort.split(','):
			if self.db.Movie.c.has_key(i):
				sort_columns.append(col(i))
	else:
		sort_columns = [col('number')]

	statement = select(columns=columns, order_by=sort_columns)

	for i in where:
		if i in ('seen', 'loaned'):	# force boolean
			if where[i].lower() in ('0', 'no', 'n'):
				where[i] = False
			else:
				where[i] = True
		if i in ('year', 'number', 'runtime', 'seen', 'loaned', 'rating'):
			statement.append_whereclause(col(i)==where[i])
		else:
			statement.append_whereclause(col(i).like('%' + where[i] + '%' ))

	movies = statement.execute().fetchall()
	if not movies:
		print _("No movie found")
	else:
		for movie in movies:
			print "\033[31;1m[%s]\033[0m\t\033[38m%s\033[0m (\033[35m%s\033[0m), %s - \033[32m%s\033[0m" % \
				(movie.number, movie.title, movie.o_title, movie.year, movie.director)
	sys.exit()

def check_dependencies():
	ostype = None
	if sys.version.rfind('Debian'):
		ostype = 'debian'

	(missing, extra) = gutils.get_dependencies()

	def __print_missing(modules):
		import string
		missing = ''
		for i in modules:
			if i['version']==False or (not isinstance(i['version'], bool) and i['version'].startswith('-')):
				tmp = None
				if ostype is not None:
					if ostype == 'debian' and i.has_key('debian'):
						tmp = "\n%s package" % i['debian']
						if i.has_key('debian_req') and i['debian_req'] is not None:
							tmp += "\n\tminimum required package version: %s" % i['debian_req']
				if tmp is None:
					tmp = "\n%s module" % i['module']
					if i.has_key('module_req') and i['module_req'] is not None:
						tmp += "\n\tminimum required module version: %s" % i['module_req']
					if i.has_key('url'):
						tmp += "\n\tURL: %s" % i['url']
				if i['version'] is not False and i['version'].startswith('-'):
					tmp += "\n\tavailable module version: %s" % i['version'][1:]
				if tmp is not None:
					missing += tmp
		if missing == '':
			return None
		else:
			return missing

	tmp = __print_missing(missing)
	if tmp is not None:
		print 'Dependencies missing:'
		print '===================='
		print tmp
	tmp = __print_missing(extra)
	if tmp is not None:
		print '\n\nOptional dependencies missing:'
		print '============================='
		print tmp, "\n"

def show_dependencies():
	(missing, extra) = gutils.get_dependencies()
	for i in missing:
		print "%(module)s :: %(version)s" % i
	for i in extra:
		print "%(module)s :: %(version)s" % i

def con_usage():
	print "USAGE:", sys.argv[0], '[OPTIONS]'
	print "\nOPTIONS:"
	print "-h, --help\tprints this screen"
	print "-D, --debug\trun with more debug info"
	print "-C, --clean\tfind and delete orphan files in posters directory"
	print "--check-dep\tcheck dependencies"
	print "--show-dep\tshow dependencies"
	print "--sqlecho\tprint SQL queries"
	print "--home DIR \tset Griffith's home directory (instead of the default ~/.griffith)"
	print "\n printing movie list:"
	print "-c <expr>, --cast=<expr>"
	print "-d <expr>, --director=<expr>"
	print "-o <expr>, --original_title=<expr>"
	print "-t <expr>, --title=<expr>"
	print "-y <number>, --year=<number>"
	print "-s <columns>, --sort=<columns>"




#!/usr/bin/env python
# -*- coding: utf-8 -*-

__revision__ = '$Id: winsetup.py 968 2008-07-26 20:17:42Z piotrek $'

# Copyright (c) 2005-2007 Vasco Nunes
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import time
import sys

# ModuleFinder can't handle runtime changes to __path__, but win32com uses them

try:
    import modulefinder
    import win32com
    for p in win32com.__path__[1:]:
        modulefinder.AddPackagePath("win32com", p)
    for extra in ["win32com.shell"]: #,"win32com.mapi"
        __import__(extra)
        m = sys.modules[extra]
        for p in m.__path__[1:]:
            modulefinder.AddPackagePath(extra, p)
except ImportError:
    # no build path setup, no worries.
    pass

from distutils.core import setup
import glob
import py2exe

opts = {
    "py2exe": {
        "includes": "pygtk,cairo,pangocairo,gobject,cgi,PIL,pysqlite2,pysqlite2.*,pango,atk,gobject,tempfile,csv,xml.dom,xml.dom.ext,xml.dom.minidom,xml.sax,threading,htmlentitydefs,sqlalchemy,sqlalchemy.*,sqlalchemy.mods.*,sqlalchemy.databases.*,sqlalchemy.engine.*,sqlalchemy.ext.*,sqlalchemy.orm.*,zipfile,webbrowser,shutil,reportlab,reportlab.pdfgen,reportlab.pdfgen.canvas,reportlab.platypus,reportlab.pdfbase.ttfonts,smtplib,win32com,platform,winshell,psycopg2,MySQLdb,chardet,gzip,commands,encodings,encodings.*",
        "optimize": 2,
		"dist_dir": "dist",
    }
}

setup(
    name = "Griffith",
    version = "0.9.8",
	description = 'Griffith - A film manager',
    author = 'Vasco Nunes/Piotr Ozarowski',
    author_email = 'vasco.m.nunes@gmail.com',
    url = 'http://www.griffith.cc/',
    license = 'GPL',
    windows = [
        {
            "script": "griffith",
            "icon_resources": [(1, "images\griffith.ico")]
        }],
	options = opts,
		data_files=[
		("images", glob.glob("data\\*.png")),
		("images/export_templates", glob.glob("export_templates\\*.dtd")),
		("images/export_templates/csv",	glob.glob("export_templates\\csv\\*.*")),
		("images/export_templates/xml",	glob.glob("export_templates\\xml\\*.*")),
		("images/export_templates/latex", glob.glob("export_templates\\latex\\*.*")),
		("images/export_templates/html_table", glob.glob("export_templates\\html_table\\*.*")),
		("images/export_templates/html_tables",	glob.glob("export_templates\\html_tables\\*.*")),
		("glade", glob.glob("glade\\*.*")),
		("i18n/bg/LC_MESSAGES", glob.glob("i18n\\bg\\LC_MESSAGES\\*.mo")),
		("i18n/ca/LC_MESSAGES",	glob.glob("i18n\\ca\\LC_MESSAGES\\*.mo")),
		("i18n/cs/LC_MESSAGES",	glob.glob("i18n\\cs\\LC_MESSAGES\\*.mo")),
		("i18n/da/LC_MESSAGES",	glob.glob("i18n\\da\\LC_MESSAGES\\*.mo")),
		("i18n/de/LC_MESSAGES",	glob.glob("i18n\\de\\LC_MESSAGES\\*.mo")),
		("i18n/el/LC_MESSAGES",	glob.glob("i18n\\el\\LC_MESSAGES\\*.mo")),
		("i18n/en_GB/LC_MESSAGES", glob.glob("i18n\\en_GB\\LC_MESSAGES\\*.mo")),
		("i18n/es/LC_MESSAGES",	glob.glob("i18n\\es\\LC_MESSAGES\\*.mo")),
		("i18n/et/LC_MESSAGES",	glob.glob("i18n\\et\\LC_MESSAGES\\*.mo")),
		("i18n/fa/LC_MESSAGES",	glob.glob("i18n\\fa\\LC_MESSAGES\\*.mo")),
		("i18n/fi/LC_MESSAGES",	glob.glob("i18n\\fi\\LC_MESSAGES\\*.mo")),
		("i18n/fr/LC_MESSAGES",	glob.glob("i18n\\fr\\LC_MESSAGES\\*.mo")),
		("i18n/hu/LC_MESSAGES",	glob.glob("i18n\\hu\\LC_MESSAGES\\*.mo")),
		("i18n/id/LC_MESSAGES",	glob.glob("i18n\\id\\LC_MESSAGES\\*.mo")),
		("i18n/it/LC_MESSAGES",	glob.glob("i18n\\it\\LC_MESSAGES\\*.mo")),
		("i18n/ja/LC_MESSAGES",	glob.glob("i18n\\ja\\LC_MESSAGES\\*.mo")),
		("i18n/ko/LC_MESSAGES",	glob.glob("i18n\\ko\\LC_MESSAGES\\*.mo")),
		("i18n/lv/LC_MESSAGES",	glob.glob("i18n\\lv\\LC_MESSAGES\\*.mo")),
		("i18n/nb/LC_MESSAGES",	glob.glob("i18n\\nb\\LC_MESSAGES\\*.mo")),
		("i18n/nds/LC_MESSAGES",	glob.glob("i18n\\nds\\LC_MESSAGES\\*.mo")),
		("i18n/nl/LC_MESSAGES",	glob.glob("i18n\\nl\\LC_MESSAGES\\*.mo")),
		("i18n/oc/LC_MESSAGES",	glob.glob("i18n\\oc\\LC_MESSAGES\\*.mo")),
		("i18n/pl/LC_MESSAGES",	glob.glob("i18n\\pl\\LC_MESSAGES\\*.mo")),
		("i18n/ps/LC_MESSAGES",	glob.glob("i18n\\ps\\LC_MESSAGES\\*.mo")),
		("i18n/pt/LC_MESSAGES",	glob.glob("i18n\\pt\\LC_MESSAGES\\*.mo")),
		("i18n/pt_BR/LC_MESSAGES", glob.glob("i18n\\pt_BR\\LC_MESSAGES\\*.mo")),
		("i18n/ru/LC_MESSAGES",	glob.glob("i18n\\ru\\LC_MESSAGES\\*.mo")),
		("i18n/sk/LC_MESSAGES",	glob.glob("i18n\\sk\\LC_MESSAGES\\*.mo")),
		("i18n/sr/LC_MESSAGES",	glob.glob("i18n\\sr\\LC_MESSAGES\\*.mo")),
		("i18n/sv/LC_MESSAGES",	glob.glob("i18n\\sv\\LC_MESSAGES\\*.mo")),
		("i18n/tr/LC_MESSAGES",	glob.glob("i18n\\tr\\LC_MESSAGES\\*.mo")),
		("i18n/uk/LC_MESSAGES",	glob.glob("i18n\\uk\\LC_MESSAGES\\*.mo")),
		("i18n/zh_CN/LC_MESSAGES", glob.glob("i18n\\zh_CN\\LC_MESSAGES\\*.mo")),
		("lib/plugins", glob.glob("lib\\plugins\\*.*")),
		("lib/plugins/export", glob.glob("lib\\plugins\\export\\*.*")),
		("lib/plugins/movie", glob.glob("lib\\plugins\\movie\\*.*")),
		("lib/plugins/imp", glob.glob("lib\\plugins\\imp\\*.*")),
		("images", glob.glob("images\\*.png")),
		("lib", glob.glob("lib\\*.*"))],
)



# -*- coding: utf-8 -*-

__revision__ = '$Id: PluginMovieAniDB.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2005-2008 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils, movie
import string, re
from gutils import decompress

plugin_name         = 'AnimeDB'
plugin_description  = 'Anime DataBase'
plugin_url          = 'www.anidb.net'
plugin_language     = _('English')
plugin_author       = 'Piotr Ożarowski'
plugin_author_email = '<ozarow+griffith@gmail.com>'
plugin_version      = '2.5'

aid_pattern = re.compile('[?&;]aid=(\d+)')

class Plugin(movie.Movie):
	def __init__(self, id):
		self.encode = 'utf-8'
		if string.find(id, 'http://') != -1:
			self.url = str(id)
			self.movie_id = 'anidb'
		else:
			self.movie_id = str(id)
			self.url = "http://anidb.net/perl-bin/animedb.pl?show=anime&aid=%s" % self.movie_id

	def initialize(self):
		self.page = decompress(self.page)
		if self.movie_id == 'anidb':
			aid =  aid_pattern.search(self.page)
			if aid:
				self.movie_id = aid.groups()[0]
				self.url = "http://anidb.net/perl-bin/animedb.pl?show=anime&aid=%s" % self.movie_id
			else:
				return False
		self.page = gutils.after(self.page, 'id="layout-content"')
		pos = string.find(self.page, 'class="g_section anime_episodes">')
		if pos >0:
			self.page = self.page[:pos]

	def get_image(self):
		match = re.search('http://img\d*.anidb.net/pics/anime/\d*.jpg', self.page)
		if match is not None:
			self.image_url = match.group()
		else:
			self.image_url = ''

	def get_o_title(self):
		self.o_title = gutils.trim(self.page, '<h1>Anime: ', '</h1>')

	def get_title(self):
		self.title = gutils.trim(self.page, '"field">Official Title', '</td>')
		self.title = gutils.trim(self.title, '<span>', '</span>')

	def get_director(self):
		self.director = ''

	def get_plot(self):
		self.plot = gutils.trim(self.page, 'class="desc">', '</div>')

	def get_year(self):
		self.year = gutils.trim(self.page, '"field">Year', '</td>')
		self.year = gutils.after(self.year, '"value">')[-4:]

	def get_runtime(self):
		self.runtime = ''

	def get_genre(self):
		self.genre = gutils.trim(self.page, '>Categories<', '</td>')
		self.genre = gutils.after(self.genre, 'value">')
		self.genre = gutils.strip_tags(self.genre)
		if len(self.genre) and self.genre.endswith('- similar'):
			self.genre =  self.genre[:-9]
		elif self.genre == '-':
			self.genre = ''
		self.genre = string.replace(self.genre, '\n', '')

	def get_cast(self):
		self.cast = ''

	def get_classification(self):
		self.classification = ''

	def get_studio(self):
		self.studio = gutils.trim(self.page, '"field">Producers', '</td>')
		self.studio = gutils.strip_tags(self.studio)
		if len(self.studio) and self.studio[:2] == " (":
			self.studio = self.studio[2:]
			if self.studio[len(self.studio)-1:] == ')':
				self.studio = self.studio[:len(self.studio)-1]
		self.studio = string.replace(self.studio, '\n', '')

	def get_o_site(self):
		self.o_site = gutils.trim(self.page, '"field">URL', '</td>')
		self.o_site = gutils.trim(self.o_site, 'href="', '"')

	def get_site(self):
		self.site = self.url

	def get_trailer(self):
		self.trailer = ''

	def get_country(self):
		self.country = ''

	def get_rating(self):
		self.rating = gutils.trim(self.page, '"field">Rating', '</td>')

	def get_notes(self):
		self.notes = ''
		# ...type
		atype = gutils.trim(self.page, '"field">Type', '</td>')
		atype = gutils.clean(atype)
		if atype != '':
			self.notes += "Type: %s\n" % atype
		# ...number of episodes
		episodes = gutils.trim(self.page, '"field">Episodes', '</td>')
		episodes = gutils.clean(episodes)
		if episodes != '':
			self.notes += "Episodes: %s\n" % episodes

class SearchPlugin(movie.SearchMovie):
	def __init__(self):
		self.encode = 'utf-8'
		self.original_url_search	= 'http://anidb.net/perl-bin/animedb.pl?show=animelist&do.search=search&adb.search='
		self.translated_url_search	= 'http://anidb.net/perl-bin/animedb.pl?show=animelist&do.search=search&adb.search='

	def search(self,parent_window):
		self.open_search(parent_window)
		self.page = decompress(self.page)

		tmp = string.find(self.page, '<h1>Anime List - Search for: ')
		if tmp == -1:		# already a movie page
			self.page = ''
		else:			# multiple matches
			self.page = gutils.trim(self.page, 'class="anime_list"', '</table>');
			self.page = gutils.after(self.page, '</tr>');

		return self.page

	def get_searches(self):
		if self.page == '':	# already a movie page
			self.number_results = 1
			self.ids.append(self.url)
			self.titles.append(self.title)
		else:			# multiple matches
			elements = string.split(self.page,"</tr>")
			self.number_results = elements[-1]

			if len(elements[0]):
				for element in elements:
					element = gutils.trim(element, '<td', '</td>')
					aid = aid_pattern.search(element)
					if not aid:
						continue
					self.ids.append(aid.groups()[0])
					element = gutils.after(element, '">')
					element = gutils.strip_tags(element)
					self.titles.append(element)
			else:
				self.number_results = 0




# -*- coding: UTF-8 -*-

__revision__ = '$Id: about.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2005-2008 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gtk
import version
import os
import sys

class AboutDialog:
	"""Shows a gtk about dialog"""
	def __init__(self, locations):
		TRANSLATORS_FILE = os.path.join(locations['share'], 'TRANSLATORS') # remember to encode this file in UTF-8
		IMAGES_DIR = locations['images']
		dialog = gtk.AboutDialog()
		dialog.set_name(version.pname)
		dialog.set_version(version.pversion)
		dialog.set_copyright("Copyright © 2005-2008 Vasco Nunes. Piotr Ożarowski")
		dialog.set_website(version.pwebsite)
		dialog.set_authors([
			_("Main Authors") + ':',
			version.pauthor.replace(', ', '\n') + "\n",
			_("Programmers") + ':',
			'Jessica Katharina Parth <Jessica.K.P@women-at-work.org>',
			'Michael Jahn <mikej06@hotmail.com>\n',
			_('Contributors:'), # FIXME: remove ":"
			'Christian Sagmueller <christian@sagmueller.net>\n' \
			'Arjen Schwarz <arjen.schwarz@gmail.com>'
		])
		dialog.set_artists([_("Logo, icon and general artwork " + \
			"by Peek <peekpt@gmail.com>." + \
			"\nPlease visit http://www.peekmambo.com/\n"),
			'seen / unseen icons by dragonskulle <dragonskulle@gmail.com>'
		])
		data = None
		if os.path.isfile(TRANSLATORS_FILE):
			data = open(TRANSLATORS_FILE).read()
		elif os.path.isfile(TRANSLATORS_FILE+'.gz'):
			from gutils import decompress
			data = decompress(open(TRANSLATORS_FILE + '.gz').read())
		elif os.name == 'posix':
			if os.path.isfile('/usr/share/doc/griffith/TRANSLATORS'):
				data = open('/usr/share/doc/griffith/TRANSLATORS').read()
			elif os.path.isfile('/usr/share/doc/griffith/TRANSLATORS.gz'):
				from gutils import decompress
				data = decompress(open('/usr/share/doc/griffith/TRANSLATORS.gz').read())
		translator_credits = ''
		if data:
			for line in data.split('\n'):
				if line.startswith('* '):
					lang = line[2:]
					if _(lang) != lang:
						line = "* %s:" % _(lang)
				translator_credits += "%s\n" % line
		else:
			translator_credits = _("See TRANSLATORS file")
		dialog.set_translator_credits(translator_credits)
		logo_file = os.path.abspath(os.path.join(IMAGES_DIR, 'griffith.png'))
		logo = gtk.gdk.pixbuf_new_from_file(logo_file)
		dialog.set_logo(logo)
		if os.path.isfile('/usr/share/common-licenses/GPL-2'):
			dialog.set_license(open('/usr/share/common-licenses/GPL-2').read())
		else:
			dialog.set_license(_("This program is released under the GNU" + \
				"General Public License.\n" + \
				"Please visit http://www.gnu.org/copyleft/gpl.html for details."))
		dialog.set_comments(version.pdescription)
		dialog.run()
		dialog.destroy()



# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovieDVDEmpire.py 1040 2008-11-15 21:13:49Z mikej06 $'

# Copyright (c) 2007
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import re

plugin_name = "DVD Empire"
plugin_description = "International Retailer of DVD Movies"
plugin_url = "www.dvdempire.com"
plugin_language = _("English")
plugin_author = "Michael Jahn"
plugin_author_email = "<mikej06@hotmail.com>"
plugin_version = "1.1"

class Plugin(movie.Movie):
	def __init__(self, id):
		self.encode='iso-8859-1'
		self.movie_id = id
		self.url = "http://www.dvdempire.com/Exec/v4_item.asp?item_id=" + str(self.movie_id)
	
	def get_image(self):
		tmp_page = gutils.trim(self.page, '<td align="center" valign="top" class="fontsmall">', '</td>')
		self.image_url = gutils.trim(tmp_page, '<img src=\'', '\' ')

	def get_o_title(self):
		self.o_title = gutils.strip_tags(gutils.trim(self.page,'<td class="fontxlarge" valign="top" align="left">', '<nobr>'))

	def get_title(self):
		self.title = gutils.strip_tags(gutils.trim(self.page,'<td class="fontxlarge" valign="top" align="left">', '<nobr>'))

	def get_director(self):
		self.director = gutils.strip_tags(gutils.trim(self.page,">Directors:","</a>"))
		self.director = self.director.replace('&nbsp;', '')
		self.director = self.director.replace('&#149;', '')

	def get_plot(self):
		self.plot = gutils.strip_tags(gutils.trim(self.page, '<td width="100%" valign="top" class="fontsmall3">', '</td>'))
		self.plot = self.plot.replace('\x93', '"')

	def get_year(self):
		self.year = gutils.strip_tags(gutils.trim(self.page, '>Production Year:', '<br />'))

	def get_runtime(self):
		self.runtime = gutils.strip_tags(gutils.trim(self.page, '>Length:', '<br />'))

	def get_genre(self):
		self.genre = gutils.strip_tags(gutils.trim(self.page, '>Genre</b>:', '</tr>'))

	def get_cast(self):
		self.cast = gutils.trim(self.page, '>Actors:', '</td><td')
		self.cast = self.cast.replace('<br>', '\n')
		self.cast = self.cast.replace('<br />', '\n')
		self.cast = self.cast.replace('&nbsp;', '')
		self.cast = self.cast.replace('&#8226;', '')
		self.cast = self.cast.replace('&#149;', '')
		self.cast = gutils.strip_tags(self.cast)

	def get_classification(self):
		self.classification = gutils.strip_tags(gutils.trim(self.page, '>Rating:', '<br />'))

	def get_studio(self):
		self.studio = gutils.strip_tags(gutils.trim(self.page, '>Studio:', '<br />'))

	def get_o_site(self):
		self.o_site = ""

	def get_site(self):
		self.site = self.url

	def get_trailer(self):
		self.trailer = ""

	def get_country(self):
		self.country = ""

	def get_rating(self):
		self.rating = gutils.clean(gutils.trim(self.page, '>Overall Rating:', ' out of'))
		try:
			tmp_float = float(self.rating)
			tmp_float = round(2 * tmp_float, 0)
			self.rating = str(tmp_float)
		except:
			self.rating = '0'

	def get_notes(self):
		self.notes = ''
		tmp_page = gutils.trim(self.page, 'Features:', '<b>')
		tmp_page = tmp_page.replace('<br>', '\n')
		tmp_page = tmp_page.replace('<br />', '\n')
		tmp_page = gutils.strip_tags(tmp_page)
		if tmp_page <> '':
			self.notes = self.notes + '\nFeatures:' + tmp_page + '\n'
		tmp_page = gutils.trim(self.page, 'Video:', '<b>')
		tmp_page = tmp_page.replace('<br>', '\n')
		tmp_page = tmp_page.replace('<br />', '\n')
		tmp_page = gutils.strip_tags(tmp_page)
		if tmp_page <> '':
			self.notes = self.notes + '\nVideo:' + tmp_page + '\n'
		tmp_page = gutils.trim(self.page, 'Audio:', '<b>')
		tmp_page = tmp_page.replace('<br>', '\n')
		tmp_page = tmp_page.replace('<br />', '\n')
		tmp_page = gutils.strip_tags(tmp_page)
		if tmp_page <> '':
			self.notes = self.notes + '\nAudio:' + tmp_page
		tmp_page = gutils.trim(self.page, 'Subtitles:', '<b>')
		tmp_page = gutils.strip_tags(tmp_page)
		if tmp_page <> '':
			self.notes = self.notes + '\nSubtitles:\n' + tmp_page + '\n'

class SearchPlugin(movie.SearchMovie):
	def __init__(self):
		self.original_url_search   = "http://www.dvdempire.com/Exec/v1_search_all.asp?&site_media_id=0&pp=&search_refined=32&used=0&string="
		self.translated_url_search = "http://www.dvdempire.com/Exec/v1_search_all.asp?&site_media_id=0&pp=&search_refined=32&used=0&string="
		self.encode = 'iso-8859-1'

	def search(self,parent_window):
		self.open_search(parent_window)
		# short the content
		tmp_page = gutils.trim(self.page,'<select name="sort"', 'Click Here to make a Suggestion</a>')
		#
		# try to get all result pages (not so nice, but it works)
		#
		tmp_pagecount = gutils.trim(self.page, '<div id="Search_Container" name="Search_Container">', '</table>')
		tmp_pagecountintuse = 1
		elements = tmp_pagecount.split("&page=")
		for element in elements:
			try:
				tmp_pagecountint = int(gutils.before(element, '\''))
			except:
				tmp_pagecountint = 0
			if tmp_pagecountint > tmp_pagecountintuse:
				tmp_pagecountintuse = tmp_pagecountint
		tmp_pagecountintcurrent = 1
		while tmp_pagecountintuse > tmp_pagecountintcurrent and tmp_pagecountintuse < 4:
			tmp_pagecountintcurrent = tmp_pagecountintcurrent + 1
			self.url = "http://www.dvdempire.com/Exec/v1_search_all.asp?&site_media_id=0&pp=&search_refined=32&used=0&page=" + str(tmp_pagecountintcurrent) + "&string="
			self.open_search(parent_window)
			tmp_page2 = gutils.trim(self.page,'<select name="sort"', 'Click Here to make a Suggestion</a>')
			tmp_page = tmp_page + tmp_page2

		self.page = tmp_page
		return self.page

	def get_searches(self):
		split_pattern = re.compile('<a[\t ]+href=["\']/Exec/v4_item[.]asp[?]userid=[-0-9]+[&]amp;item_id=')
		check_pattern = re.compile('[0-9]+[&]amp;searchID=[0-9]+["\']>')
		elements = split_pattern.split(self.page)
		elements[0] = ''
		for element in elements:
			if element <> '':
				is_an_item = check_pattern.search(element)
				if is_an_item:
					tmp_title = gutils.strip_tags(gutils.trim(element, ">", "</a>"))
					if tmp_title <> '':
						tmp_id = gutils.before(element,'&')
						self.ids.append(tmp_id)
						self.titles.append(tmp_title)
